@@ -19,6 +19,7 @@
 from contextlib import closing
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+import pandas as pd
 import pyexasol
 from pyexasol import ExaConnection
 
@@ -63,20 +64,20 @@ class ExasolHook(DbApiHook):
         conn = pyexasol.connect(**conn_args)
         return conn
 
-    def get_pandas_df(self, sql: Union[str, list], parameters: Optional[dict] = None, **kwargs) -> None:
+    def get_pandas_df(
+        self, sql: Union[str, list], parameters: Optional[dict] = None, **kwargs
+    ) -> pd.DataFrame:
         """
         Executes the sql and returns a pandas dataframe
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         :param kwargs: (optional) passed into pyexasol.ExaConnection.export_to_pandas method
-        :type kwargs: dict
         """
         with closing(self.get_conn()) as conn:
-            conn.export_to_pandas(sql, query_params=parameters, **kwargs)
+            df = conn.export_to_pandas(sql, query_params=parameters, **kwargs)
+            return df
 
     def get_records(
         self, sql: Union[str, list], parameters: Optional[dict] = None
@@ -86,9 +87,7 @@ class ExasolHook(DbApiHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         """
         with closing(self.get_conn()) as conn:
             with closing(conn.execute(sql, parameters)) as cur:
@@ -100,9 +99,7 @@ class ExasolHook(DbApiHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
         """
         with closing(self.get_conn()) as conn:
             with closing(conn.execute(sql, parameters)) as cur:
@@ -119,15 +116,11 @@ class ExasolHook(DbApiHook):
         Exports data to a file.
 
         :param filename: Path to the file to which the data has to be exported
-        :type filename: str
         :param query_or_table: the sql statement to be executed or table name to export
-        :type query_or_table: str
         :param query_params: Query parameters passed to underlying ``export_to_file``
             method of :class:`~pyexasol.connection.ExaConnection`.
-        :type query_params: dict
         :param export_params: Extra parameters passed to underlying ``export_to_file``
             method of :class:`~pyexasol.connection.ExaConnection`.
-        :type export_params: dict
         """
         self.log.info("Getting data from exasol")
         with closing(self.get_conn()) as conn:
@@ -139,7 +132,9 @@ class ExasolHook(DbApiHook):
             )
         self.log.info("Data saved to %s", filename)
 
-    def run(self, sql: Union[str, list], autocommit: bool = False, parameters: Optional[dict] = None) -> None:
+    def run(
+        self, sql: Union[str, list], autocommit: bool = False, parameters: Optional[dict] = None, handler=None
+    ) -> Optional[list]:
         """
         Runs a command or a list of commands. Pass a list of sql
         statements to the sql parameter to get them to execute
@@ -147,16 +142,19 @@ class ExasolHook(DbApiHook):
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
-        :type sql: str or list
         :param autocommit: What to set the connection's autocommit setting to
             before executing the query.
-        :type autocommit: bool
         :param parameters: The parameters to render the SQL query with.
-        :type parameters: dict or iterable
+        :param handler: The result handler which is called with the result of each statement.
         """
         if isinstance(sql, str):
             sql = [sql]
 
+        if sql:
+            self.log.debug("Executing %d statements against Exasol DB", len(sql))
+        else:
+            raise ValueError("List of SQL statements is empty")
+
         with closing(self.get_conn()) as conn:
             if self.supports_autocommit:
                 self.set_autocommit(conn, autocommit)
@@ -164,20 +162,29 @@ class ExasolHook(DbApiHook):
             for query in sql:
                 self.log.info(query)
                 with closing(conn.execute(query, parameters)) as cur:
+                    results = []
+
+                    if handler is not None:
+                        cur = handler(cur)
+
+                    for row in cur:
+                        self.log.info("Statement execution info - %s", row)
+                        results.append(row)
+
                     self.log.info(cur.row_count)
             # If autocommit was set to False for db that supports autocommit,
-            # or if db does not supports autocommit, we do a manual commit.
+            # or if db does not support autocommit, we do a manual commit.
             if not self.get_autocommit(conn):
                 conn.commit()
 
+        return results
+
     def set_autocommit(self, conn, autocommit: bool) -> None:
         """
         Sets the autocommit flag on the connection
 
         :param conn: Connection to set autocommit setting to.
-        :type conn: connection object
         :param autocommit: The autocommit setting to set.
-        :type autocommit: bool
         """
         if not self.supports_autocommit and autocommit:
             self.log.warning(
@@ -194,7 +201,6 @@ class ExasolHook(DbApiHook):
         does not support autocommit.
 
         :param conn: Connection to get autocommit setting from.
-        :type conn: connection object
         :return: connection autocommit setting.
         :rtype: bool
         """
@@ -210,9 +216,7 @@ class ExasolHook(DbApiHook):
         hence we return cell without any conversion.
 
         :param cell: The cell to insert into the table
-        :type cell: object
         :param conn: The database connection
-        :type conn: connection object
         :return: The cell
         :rtype: object
         """
